@@ -19,6 +19,6 @@ def build_wheel(wheel_directory, config_settings=None, metadata_directory=None):
         msg = 'some-file-that-is-needed-for-build.txt is missing!'
         raise FileNotFoundError(msg)
     # pragma: no cover
-    file = 'test_cant_build_via_sdist-1.0.0-py2.py3-none-any.whl'
+    file = 'test_cant_build_via_sdist-1.0.0-py3-none-any.whl'
     zipfile.ZipFile(os.path.join(wheel_directory, file), 'w').close()
     return file
